/**
 * @file tz.h
 * @brief Implements time zone constants.
 * @note Copyright (C) 2021 Richard Cochran <richardcochran@gmail.com>
 * @note SPDX-License-Identifier: GPL-2.0+
 */
#ifndef HAVE_TZ_H
#define HAVE_TZ_H

#define MAX_TZ_DISPLAY_NAME	10
#define MAX_TIME_ZONES		4

#endif
